use flair::flair;
use ssw::websrv::WebsrvConfig;
use ssw::websrv::run_server;
use std::fs;
use std::collections::HashMap;

//1 - Ingest all the pages into a variable
// : all the pages must start with a number indicating their order
//2 - build the index with the pages
//3 - Put all the generated content inside the html folder
//4 - Start the server and serve

//#1 : Sends (pages_summary, pages_content)
fn ingest_pages(pages_path: &str) -> Option<(String, String)> {

    //Getting the name of existing pages
    let mut pages: Vec<String> = Vec::new();
    for file in fs::read_dir(pages_path).unwrap() {
        //println!("{:#?}", file.unwrap().path());
        let mypage = String::from(file
                                  .unwrap()
                                  .path()
                                  .file_name()
                                  .unwrap()
                                  .to_str()
                                  .unwrap()
            );

        &mut pages.push(mypage);
    }
    pages.sort();
    pages.reverse();

    if pages.len() > 0 {
        //Handling content
        let mut pages_content = String::new();
        let mut page_sum = String::new();
        for page in pages {
            let vars: HashMap<String, String> = HashMap::from([
                (String::from("header"), fs::read_to_string("./html/header.html").unwrap()),
                (String::from("footer"), fs::read_to_string("./html/footer.html").unwrap()),
            ]);
            let fullpath = String::from(format!("{pages_path}/{page}"));
            let page_res = flair::analyze_file(&fullpath, vars).unwrap();
            //TODO Write the pages to their own file
            //and put the correct address in %%sum%%
            //So the index is simply "latest articles

            page_sum.push_str(&page_link(&page, &page_res));
            &mut pages_content.push_str(&page_res);
        }
        return Some((page_sum, pages_content));
    } else {
        return None;
    }
}

fn page_link(page_name: &str, content: &String) -> String {
    let page_html = str::replace(page_name, ".ft", "");
    let page_loc = format!("./html/pages/{page_html}");
    let page_name_short = str::replace(page_name, ".html.ft", "");
    let page_link = format!("<a href=\"pages/{page_html}\">{page_name_short}</a><br>\n");

    fs::write(page_loc, content);
    return String::from(page_link);
}


//#2 & #3
fn generate_index(pages: (String, String), index_path: &str, html_path: &str) {
    let vars: HashMap<String, String> = HashMap::from([
        (String::from("header"), fs::read_to_string("./html/header.html").unwrap()),
        (String::from("footer"), fs::read_to_string("./html/footer.html").unwrap()),
        (String::from("pages"), pages.1),
        (String::from("sum"), pages.0),
    ]);

    let index = flair::analyze_file("./templates/index.html.ft", vars).unwrap();
    
    fs::write("./html/index.html", index.as_str()).expect("Could not write index");
}

//#4
fn start_websrv(conf: WebsrvConfig) {
    run_server(conf);
    loop {
        continue;
    }
}

fn main() {
    let pages = match ingest_pages("./templates/pages") {
        Some(p) => p,
        None => (String::new(), String::new()),
    };

    generate_index(pages, "./templates/index.html.ft", "./html");

    //Web conf
    let webconf = WebsrvConfig {
        nbr_of_threads: 10,
        bind_addr: String::from("0.0.0.0:8000"),
        root_folder: String::from("./html"),
    };
    start_websrv(webconf);

}
